@@ -17,8 +17,6 @@ use std::time::Duration;
 pub struct ElectrsD {
     /// Process child handle, used to terminate the process when this struct is dropped
     process: Child,
-    /// bitcoind process connected to this electrs
-    pub bitcoind: BitcoinD,
     /// Electrum client connected to the electrs process
     pub client: RawClient<ElectrumPlaintextStream>,
     /// DB directory, where electrs store indexes. It is kept in the struct so that
@@ -35,25 +33,30 @@ pub struct ElectrsD {
 pub enum Error {
     /// Wrapper of io Error
     Io(std::io::Error),
+
     /// Wrapper of bitcoind Error
     Bitcoind(bitcoind::Error),
+
     /// Wrapper of electrum_client Error
     ElectrumClient(electrum_client::Error),
+
     /// Wrapper of bitcoincore_rpc Error
     BitcoinCoreRpc(bitcoind::bitcoincore_rpc::Error),
+
+    #[cfg(feature = "trigger")]
+    /// Wrapper of nix Error
+    Nix(nix::Error),
 }
 
 impl ElectrsD {
     /// Create a new electrs process connected with the given bitcoind
     pub fn new<S: AsRef<OsStr>>(
         exe: S,
-        bitcoind: BitcoinD,
+        bitcoind: &BitcoinD,
         view_stderr: bool,
         http_enabled: bool,
     ) -> Result<ElectrsD, Error> {
-        let mut args = vec![];
-
-        args.push("-vvv");
+        let mut args = vec!["-vvv"];
 
         let _db_dir = TempDir::new()?;
         let db_dir = format!("{}", _db_dir.path().display());
@@ -64,11 +67,12 @@ impl ElectrsD {
         args.push("regtest");
 
         args.push("--cookie-file");
-        let cookie_file = format!("{}", bitcoind.cookie_file.display());
+        let cookie_file = format!("{}", bitcoind.params.cookie_file.display());
         args.push(&cookie_file);
 
         args.push("--daemon-rpc-addr");
-        args.push(&bitcoind.rpc_url[7..]);
+        let rpc_socket = bitcoind.params.rpc_socket.to_string();
+        args.push(&rpc_socket);
 
         args.push("--jsonrpc-import");
 
@@ -86,6 +90,7 @@ impl ElectrsD {
             esplora_url_string = format!("0.0.0.0:{}", get_available_port()?);
             args.push("--http-addr");
             args.push(&esplora_url_string);
+            #[allow(clippy::redundant_clone)]
             Some(esplora_url_string.clone())
         } else {
             None
@@ -108,14 +113,22 @@ impl ElectrsD {
         };
 
         Ok(ElectrsD {
-            client,
-            bitcoind,
             process,
+            client,
             _db_dir,
             electrum_url,
             esplora_url,
         })
     }
+
+    #[cfg(feature = "trigger")]
+    /// triggers electrs sync by sending the `SIGUSR1` signal, useful to call after a block for example
+    pub fn trigger(&self) -> Result<(), Error> {
+        Ok(nix::sys::signal::kill(
+            nix::unistd::Pid::from_raw(self.process.id() as i32),
+            nix::sys::signal::SIGUSR1,
+        )?)
+    }
 }
 
 impl Drop for ElectrsD {
@@ -148,11 +161,18 @@ impl From<bitcoind::bitcoincore_rpc::Error> for Error {
     }
 }
 
+#[cfg(feature = "trigger")]
+impl From<nix::Error> for Error {
+    fn from(e: nix::Error) -> Self {
+        Error::Nix(e)
+    }
+}
+
 #[cfg(test)]
 mod test {
     use crate::ElectrsD;
     use bitcoind::bitcoincore_rpc::RpcApi;
-    use bitcoind::BitcoinD;
+    use bitcoind::{BitcoinD, Conf};
     use electrum_client::ElectrumApi;
     use std::env;
 
@@ -160,20 +180,20 @@ mod test {
     fn test_electrsd() {
         let bitcoind_exe = env::var("BITCOIND_EXE").expect("BITCOIND_EXE env var must be set");
         let electrs_exe = env::var("ELECTRS_EXE").expect("ELECTRS_EXE env var must be set");
-        let bitcoind = BitcoinD::with_args(bitcoind_exe, vec![], true, bitcoind::P2P::No).unwrap();
-        let electrsd = ElectrsD::new(electrs_exe, bitcoind, true, false).unwrap();
+        let conf = Conf {
+            view_stdout: true,
+            ..Default::default()
+        };
+        let bitcoind = BitcoinD::with_conf(&bitcoind_exe, &conf).unwrap();
+        let electrsd = ElectrsD::new(&electrs_exe, &bitcoind, true, false).unwrap();
         let header = electrsd.client.block_headers_subscribe().unwrap();
         assert_eq!(header.height, 0);
-        let address = electrsd
-            .bitcoind
-            .client
-            .get_new_address(None, None)
-            .unwrap();
-        electrsd
-            .bitcoind
-            .client
-            .generate_to_address(101, &address)
-            .unwrap();
+        let address = bitcoind.client.get_new_address(None, None).unwrap();
+        bitcoind.client.generate_to_address(101, &address).unwrap();
+
+        #[cfg(feature = "trigger")]
+        electrsd.trigger().unwrap();
+
         let header = loop {
             std::thread::sleep(std::time::Duration::from_secs(1));
             let header = electrsd.client.block_headers_subscribe().unwrap();
@@ -182,5 +202,10 @@ mod test {
             }
         };
         assert_eq!(header.height, 101);
+
+        // launch another instance to check there are no fixed port used
+        let electrsd = ElectrsD::new(&electrs_exe, &bitcoind, true, false).unwrap();
+        let header = electrsd.client.block_headers_subscribe().unwrap();
+        assert_eq!(header.height, 101);
     }
 }
